﻿local Engine, Api, Locale, Settings = unpack(Immersive)

-- Forward Declaration
local GetRect

-- Constants
local MULTIBAR_ACTION_BUTTON_COOLDOWN = "MultiBarBottomLeftButton%dCooldown"

local Default = Engine:GetModule("Default")

-- Registers the element.
function Default:RegisterMultiBarBottomLeft()
	local settings = Settings:Get(self.Name).Elements["MultibarBottomLeft"]
	local bar = Api:RegisterElement("MultibarBottomLeft", settings)
	
	-- Add frames
	bar:AddFrame(MultiBarBottomLeft)
	
	-- Add conditions
	bar:AddConditionRange(settings.Conditions)
	
	if Engine.IsClassic then
		bar:AddHotspot(MultiBarBottomLeft:GetRect())
	else
		bar.Animation:RegisterEvent("PLAYER_ENTERING_WORLD")
		bar.Animation:SetScript("OnEvent", function(self, event)
			-- Add hotspots
			bar:ClearHotspots()
			bar:AddHotspot(GetRect())
		end)
	end
	
	bar.OnFadeIn = function(self, frame, alpha)
		if not MultiBarBottomLeftButton1Cooldown:GetDrawBling() then
			for i = 1, 12 do
				local cooldown = _G[string.format(MULTIBAR_ACTION_BUTTON_COOLDOWN, i)]
				cooldown:SetDrawBling(true)
			end
		end
	end
	bar.OnFadeOut = function(self, frame, alpha)
		if alpha < 0.01 then
			for i = 1, 12 do
				local cooldown = _G[string.format(MULTIBAR_ACTION_BUTTON_COOLDOWN, i)]
				cooldown:SetDrawBling(false)
			end
		end
	end

	print(MultiBarBottomLeft:GetParent():GetName())
end

-- Unregisters the element.
function Default:UnregisterMultiBarBottomLeft()
	Api:UnregisterElement("MultibarBottomLeft")
end

-- Calculate the containable rectangle of the MainMenuBar
GetRect = function()
	local left = MultiBarBottomLeftButton1:GetLeft()
	local bottom = MultiBarBottomLeftButton1:GetBottom()
	local width = MultiBarBottomLeftButton12:GetRight() - left
	local height = MultiBarBottomLeftButton1:GetHeight()

	return left, bottom, width, height
end