﻿local Engine, Api, Locale, Settings = unpack(Immersive)

-- Forward Declaration
local GetRect

-- Constants
local MULTIBAR_ACTION_BUTTON_COOLDOWN = "MultiBarBottomRightButton%dCooldown"

local Default = Engine:GetModule("Default")

-- Registers the element.
function Default:RegisterMultiBarBottomRight()
	local settings = Settings:Get(self.Name).Elements["MultibarBottomRight"]
	local bar = Api:RegisterElement("MultibarBottomRight", settings)
	
	-- Add frames
	if Engine.IsClassic then
		bar:AddFrame(MultiBarBottomRight)
	else
		bar:AddFrame(MultiBarBottomRightButton7)
		bar:AddFrame(MultiBarBottomRightButton8)
		bar:AddFrame(MultiBarBottomRightButton9)
		bar:AddFrame(MultiBarBottomRightButton10)
		bar:AddFrame(MultiBarBottomRightButton11)
		bar:AddFrame(MultiBarBottomRightButton12)
	end
	
	-- Add conditions
	bar:AddConditionRange(settings.Conditions)
	
	if Engine.IsClassic then
		bar:AddHotspot(MultiBarBottomRight:GetRect())
	else
		bar.Animation:RegisterEvent("PLAYER_ENTERING_WORLD")
		bar.Animation:SetScript("OnEvent", function(self, event)
			-- Add hotspots
			bar:ClearHotspots()
			bar:AddHotspot(GetRect())
		end)
	end
	
	bar.OnFadeIn = function(self, frame, alpha)
		if not MultiBarBottomRightButton1Cooldown:GetDrawBling() then
			for i = 1, 12 do
				local cooldown = _G[string.format(MULTIBAR_ACTION_BUTTON_COOLDOWN, i)]
				cooldown:SetDrawBling(true)
			end
		end
	end
	bar.OnFadeOut = function(self, frame, alpha)
		if alpha < 0.01 then
			for i = 1, 12 do
				local cooldown = _G[string.format(MULTIBAR_ACTION_BUTTON_COOLDOWN, i)]
				cooldown:SetDrawBling(false)
			end
		end
	end
end

-- Unregisters the element.
function Default:UnregisterMultiBarBottomRight()
	Api:UnregisterElement("MultibarBottomRight")
end

-- Calculate the containable rectangle of the MainMenuBar
GetRect = function()
	local left = MultiBarBottomRightButton7:GetLeft()
	local bottom = MultiBarBottomRightButton7:GetBottom()
	local width = MultiBarBottomRightButton12:GetRight() - left
	local height = MultiBarBottomRightButton7:GetHeight()

	return left, bottom, width, height
end